#include "project.h"
#include "stdio.h"

#define BIT_DELAY 10
#define SW_LOW  (0u)
#define SW_HIGH (1u)

#define delay(a)  CyDelayUs(a)

char str[128] ; /* print buffer */
void print(char *str) 
{
    UART_PutString(str) ;
}

void init_hardware(void)
{
    __enable_irq(); /* Enable global interrupts. */
    UART_Start() ;
    /* Enable CM4.  CY_CORTEX_M4_APPL_ADDR must be updated if CM4 memory layout is changed. */
    Cy_SysEnableCM4(CY_CORTEX_M4_APPL_ADDR);     
}

void splash(void)
{
    sprintf(str, "Software Serial Test (%s %s)\n", __DATE__, __TIME__) ;
    print(str) ;
}


uint8_t sw_recv_byte(void)
{
    int bit_count = 8 ;
    uint8_t data = 0 ;
    
    while(Cy_GPIO_Read(SW_RX_0_PORT, SW_RX_0_NUM) == SW_HIGH) {
        delay(BIT_DELAY/2) ; 
    }
    delay(BIT_DELAY) ; 
    while(bit_count--) {
        data <<= 1 ;
        if (Cy_GPIO_Read(SW_RX_0_PORT, SW_RX_0_NUM) == SW_HIGH) {
            data |= 0x01 ;
        }
        delay(BIT_DELAY) ;
    }
    if (Cy_GPIO_Read(SW_RX_0_PORT, SW_RX_0_NUM) != SW_HIGH) {
        print("SW_RX, Stop bit missing!\n") ;
    }
    return( data ) ;
}


int main(void)
{
    uint8_t data = 0 ;
    
    init_hardware() ;
    
    splash() ;

    for(;;)
    {
        data = sw_recv_byte() ; /* this function blocks */
        sprintf(str, "\tCM0 received 0x%02X\n", data) ;
        print(str) ;
    }
}

/* [] END OF FILE */
